@@ -53,7 +53,6 @@
 
 #define DBL_CLICK_GRAB          (ButtonMotionMask | \
                                  PointerMotionMask | \
-                                 PointerMotionHintMask | \
                                  ButtonPressMask | \
                                  ButtonReleaseMask)
                                  
@@ -97,7 +96,8 @@ typeOfClick (Window w, XEvent * ev, gboolean allow_double_click)
 
     g_return_val_if_fail (ev != NULL, XFWM_BUTTON_UNDEFINED);
     g_return_val_if_fail (w != None, XFWM_BUTTON_UNDEFINED);
-
+    
+    XFlush (dpy);
     g = XGrabPointer (dpy, w, FALSE, DBL_CLICK_GRAB, GrabModeAsync,
         GrabModeAsync, None, None, ev->xbutton.time);
     if (g != GrabSuccess)
@@ -110,14 +110,13 @@ typeOfClick (Window w, XEvent * ev, gboolean allow_double_click)
     button = ev->xbutton.button;
     x = xcurrent = ev->xbutton.x_root;
     y = ycurrent = ev->xbutton.y_root;
-    t0 = ev->xbutton.time;
-    t1 = t0;
+    t0 = CurrentTime;
     total = 0;
     clicks = 1;
 
-    while ((ABS (x - xcurrent) < 1) && (ABS (y - ycurrent) < 1)
+    while ((ABS (x - xcurrent) < 2) && (ABS (y - ycurrent) < 2)
         && (total < params.dbl_click_time)
-        && ((t1 - t0) < params.dbl_click_time))
+        && ((CurrentTime - t0) < params.dbl_click_time))
     {
         g_usleep (10000);
         total += 10;
@@ -127,15 +126,6 @@ typeOfClick (Window w, XEvent * ev, gboolean allow_double_click)
             {
                 clicks++;
             }
-            t1 = ev->xbutton.time;
-        }
-        if (XCheckMaskEvent (dpy,
-                ButtonMotionMask | PointerMotionMask | PointerMotionHintMask,
-                ev))
-        {
-            xcurrent = ev->xmotion.x_root;
-            ycurrent = ev->xmotion.y_root;
-            t1 = ev->xmotion.time;
         }
         if ((XfwmButtonClickType) clicks == XFWM_BUTTON_DOUBLE_CLICK
             || (!allow_double_click
@@ -143,8 +133,14 @@ typeOfClick (Window w, XEvent * ev, gboolean allow_double_click)
         {
             break;
         }
+        if (XCheckMaskEvent (dpy, ButtonMotionMask | PointerMotionMask, ev))
+        {
+            xcurrent = ev->xmotion.x_root;
+            ycurrent = ev->xmotion.y_root;
+        }
     }
     XUngrabPointer (dpy, ev->xbutton.time);
+    XFlush (dpy);
     return (XfwmButtonClickType) clicks;
 }
 
@@ -311,7 +307,7 @@ handleKeyPress (XKeyEvent * ev)
                 }
                 break;
             case KEY_CYCLE_WINDOWS:
-                clientCycle (c);
+                clientCycle (c, (XEvent *) ev);
                 break;
             case KEY_CLOSE_WINDOW:
                 clientClose (c);
@@ -338,6 +334,12 @@ handleKeyPress (XKeyEvent * ev)
                 clientToggleSticky (c, TRUE);
                 frameDraw (c, FALSE, FALSE);
                 break;
+            case KEY_RAISE_WINDOW:
+                clientRaise (c);
+                break;
+            case KEY_LOWER_WINDOW:
+                clientLower (c);
+                break;
             case KEY_MOVE_NEXT_WORKSPACE:
                 workspaceSwitch (workspace + 1, c);
                 break;
@@ -382,7 +384,7 @@ handleKeyPress (XKeyEvent * ev)
             case KEY_CYCLE_WINDOWS:
                 if (clients)
                 {
-                    clientCycle (clients->prev);
+                    clientCycle (clients->prev, (XEvent *) ev);
                 }
                 break;
             default:
@@ -666,7 +668,22 @@ handleButtonPress (XButtonEvent * ev)
         }
         else if ((ev->button == Button3) && (state == AltMask) && (params.easy_click))
         {
-            edgeButton (c, CORNER_BOTTOM_RIGHT, ev);
+            if ((ev->x < c->width / 2) && (ev->y < c->height / 2))
+            {
+                edgeButton (c, CORNER_TOP_LEFT, ev);
+            }
+            else if ((ev->x < c->width / 2) && (ev->y > c->height / 2))
+            {
+                edgeButton (c, CORNER_BOTTOM_LEFT, ev);
+            }
+            else if ((ev->x > c->width / 2) && (ev->y < c->height / 2))
+            {
+                edgeButton (c, CORNER_TOP_RIGHT, ev);
+            }
+            else
+            {
+                edgeButton (c, CORNER_BOTTOM_RIGHT, ev);
+            }
         }
         else if (WIN_IS_BUTTON (win))
         {
@@ -769,7 +786,7 @@ handleButtonPress (XButtonEvent * ev)
             if (ev->button == Button1)
             {
                 clientSetFocus (c, TRUE, FALSE);
-                if (params.raise_on_click)
+                if ((params.raise_on_click) || !CLIENT_FLAG_TEST (c, CLIENT_FLAG_HAS_BORDER))
                 {
                     clientRaise (c);
                 }
@@ -851,7 +868,11 @@ handleMapRequest (XMapRequestEvent * ev)
             return;
         }
         clientShow (c, TRUE);
-        clientFocusNew(c);
+        if (CLIENT_FLAG_TEST (c, CLIENT_FLAG_STICKY) ||
+            (c->win_workspace == workspace))
+        {
+            clientFocusNew(c);
+        }
     }
     else
     {
@@ -893,6 +914,12 @@ handleUnmapNotify (XUnmapEvent * ev)
         return;
     }
 
+    if ((ev->event != ev->window) && (ev->event != root || !ev->send_event))
+    {
+        TRACE ("handleUnmapNotify (): Event ignored\n");
+        return;
+    }
+
     c = clientGetFromWindow (ev->window, WINDOW); 
     if (c)
     {
@@ -1039,6 +1066,22 @@ handleConfigureRequest (XConfigureRequestEvent * ev)
             ev->value_mask &= ~(CWSibling | CWStackMode);
         }
         clientCoordGravitate (c, APPLY, &wc.x, &wc.y);
+        if (CLIENT_FLAG_TEST (c, CLIENT_FLAG_FULLSCREEN))
+        {
+            int cx, cy;
+
+            /* size request from fullscreen windows get fullscreen */
+            
+            cx = frameX (c) + (frameWidth (c) / 2);
+            cy = frameY (c) + (frameHeight (c) / 2);
+
+            wc.x = MyDisplayX (cx, cy);
+            wc.y = MyDisplayY (cx, cy);
+            wc.width = MyDisplayWidth (dpy, screen, cx, cy);
+            wc.height = MyDisplayHeight (dpy, screen, cx, cy);
+
+            ev->value_mask |= (CWX | CWY | CWWidth | CWHeight);
+        }
         /* Clean up buggy requests that set all flags */
         if ((ev->value_mask & CWX) && (wc.x == c->x))
         {
@@ -1065,6 +1108,7 @@ handleConfigureRequest (XConfigureRequestEvent * ev)
             }
             constrained = TRUE;
         }
+#if 0
         /* Let's say that if the client performs a XRaiseWindow, we show the window if hidden */
         if ((ev->value_mask & CWStackMode) && (wc.stack_mode == Above))
         {
@@ -1077,6 +1121,7 @@ handleConfigureRequest (XConfigureRequestEvent * ev)
                 }
             }
         }
+#endif
         clientConfigure (c, &wc, ev->value_mask, 
                          (constrained ? CFG_CONSTRAINED : 0) | CFG_REQUEST);
     }
@@ -1139,6 +1184,32 @@ handleFocusIn (XFocusChangeEvent * ev)
     Client *c;
 
     TRACE ("entering handleFocusIn");
+    TRACE ("handleFocusOut (0x%lx) mode = %s",
+                ev->window,
+                (ev->mode == NotifyNormal) ?
+                "NotifyNormal" :
+                (ev->mode == NotifyWhileGrabbed) ?
+                "NotifyWhileGrabbed" :
+                "(unknown)");
+    TRACE ("handleFocusOut (0x%lx) detail = %s",
+                ev->window,
+                (ev->detail == NotifyAncestor) ?
+                "NotifyAncestor" :
+                (ev->detail == NotifyVirtual) ?
+                "NotifyVirtual" :
+                (ev->detail == NotifyInferior) ?
+                "NotifyInferior" :
+                (ev->detail == NotifyNonlinear) ?
+                "NotifyNonlinear" :
+                (ev->detail == NotifyNonlinearVirtual) ?
+                "NotifyNonlinearVirtual" :
+                (ev->detail == NotifyPointer) ?
+                "NotifyPointer" :
+                (ev->detail == NotifyPointerRoot) ?
+                "NotifyPointerRoot" :
+                (ev->detail == NotifyDetailNone) ?
+                "NotifyDetailNone" :
+                "(unknown)");
 
     if ((ev->mode == NotifyGrab) || (ev->mode == NotifyUngrab)
         || (ev->detail > NotifyNonlinearVirtual))
@@ -1165,6 +1236,32 @@ handleFocusOut (XFocusChangeEvent * ev)
 {
     Client *c;
     TRACE ("entering handleFocusOut");
+    TRACE ("handleFocusOut (0x%lx) mode = %s",
+                ev->window,
+                (ev->mode == NotifyNormal) ?
+                "NotifyNormal" :
+                (ev->mode == NotifyWhileGrabbed) ?
+                "NotifyWhileGrabbed" :
+                "(unknown)");
+    TRACE ("handleFocusOut (0x%lx) detail = %s",
+                ev->window,
+                (ev->detail == NotifyAncestor) ?
+                "NotifyAncestor" :
+                (ev->detail == NotifyVirtual) ?
+                "NotifyVirtual" :
+                (ev->detail == NotifyInferior) ?
+                "NotifyInferior" :
+                (ev->detail == NotifyNonlinear) ?
+                "NotifyNonlinear" :
+                (ev->detail == NotifyNonlinearVirtual) ?
+                "NotifyNonlinearVirtual" :
+                (ev->detail == NotifyPointer) ?
+                "NotifyPointer" :
+                (ev->detail == NotifyPointerRoot) ?
+                "NotifyPointerRoot" :
+                (ev->detail == NotifyDetailNone) ?
+                "NotifyDetailNone" :
+                "(unknown)");
 
     if ((ev->mode == NotifyGrab) || (ev->mode == NotifyUngrab)
         || (ev->detail != NotifyNonlinear))
@@ -1233,6 +1330,13 @@ handlePropertyNotify (XPropertyEvent * ev)
                 }
             }
         }
+        else if (ev->atom == wm_protocols)
+        {
+            TRACE
+                ("client \"%s\" (0x%lx) has received a wm_protocols notify",
+                c->name, c->window);
+            clientGetWMProtocols (c);
+        }
         else if (ev->atom == win_hints)
         {
             TRACE ("client \"%s\" (0x%lx) has received a win_hints notify",
